

import sys
import math

"""
***************************************************************************************************************************
# Funktion 'FAREY SEQUENCE  (Farey-Folge) - Best rational approximation'
# Formel und Berechnung nach https://www.johndcook.com/blog/2010/10/20/best-rational-approximation/

Function
#   a = Dezimalbruch, z.B. 0,3 ; 0,27 ; 0,88889 etc.
#       der durch Division von zwei Ganzahlen nachgebildet werden soll: 0,3 = 3/10 ; 0,27 = 27/100; 0,8889123 = 42185/47458

#   N = hoechstmoegliche Zahl jeweils fuer Nenner und Zaehler des Bruches (gewaehlt N = 1048575 = (2^20 - 1)
#       Achtung: beim Si5351 darf diese Zahl wegen der Registergroesse (20bit) hoechstens 1.048.575 betragen (2^20 - 1)

# Beispiel fuer Skript-Aufruf: "python3 Si5351-Reg.py 0 28.0093019648 736 2"
***************************************************************************************************************************
"""


#def farey (x, N):
def farey (x):
	a, b = 0, 1
	c, d = 1, 1
    

	while (b<=N and d<=N):
		mediant = float (a + c) / (b + d)
		if x == mediant:
		
			if b + d <= N:
				return (a + c, b + d)
		
			elif d > b:
				return (c, d)
		
			else:
				return (a, b)


		elif x > mediant:
			a, b = a + c, b + d

		else:
			 c, d = a + c, b + d


	if b > N:
    		return (c, d)

	else:
		return (a, b)





# Ab hier Hauptprogramm

""" Main"""
N = 1000000         #ACHTUNG: dieser Maximalwert N muss fuer eine Iteration in der uC-Software identisch verwendet werden
                    #Andernfalls unterscheiden sich die Register-Inhalte vom uC und die, die von diesem Skript angezeigt werden!!



"""
***************************************************************
E I N L E S E N    A R G U M E N T E  S K R I P T - A U F R U F
***************************************************************
"""
CLK = int(sys.argv[1])      #CLK-Output (Si5351-Breakout Board besitzt 3 CLK-Ausgaenge, somit moegliche Eingabewerte 0 bis 2)
PLLNA = float(sys.argv[2])  # 
MS_DIV = float(sys.argv[3]) #
R_DIV = int(sys.argv[4])    #R-Output-Divider (zulaessige Eingabewerte: 1, 2, 4, 8, 16....128)




"""
************************************************
'R' - O U T P U T - S T A G E - D I V I D E R   
Berechnung der 
Bits Rx_DIV[2:0]  vom R Output Stage Divider
Rx-DIV ist relevant fuer Si5351-Register 44
************************************************
"""
print ("\n\n\nWERTE EINGEGEBEN:")
print ("-----------------")
print ("CLK-Ausgang:    ", CLK)
print ("PLLN A:         ", PLLNA)
print ("MSx_DIV:        ", MS_DIV)
print ("R_DIV:          ", R_DIV)



R_DIV_Bits = 0                      #Berechnung erforderl. Bit-Muster von R_DIV fuer Reg. 44, 52,....
if R_DIV > 1:
   
    while R_DIV != 1:
        R_DIV = R_DIV >> 1
        R_DIV_Bits += 1
    
    R_DIV_Bits = R_DIV_Bits << 4    #Resultat erforderl. Bit-Muster von R_DIV fuer Reg. 44, 52,....




"""
***************************************************************
F E E D B A C K  M U L T I S Y N T H  P L L  D I V I D E R MSNA
Berechnung
a, b, c  von MSNA
MSNA_P1, MSNA_P2, MSNA_P3
Werte fuer MSNA-Register Register 26 - 33
****************************************************************
"""


a = math.floor(PLLNA)

x = PLLNA - a           #Nachkommastellen
(b, c) =  farey (x)     #Berechnung von Zaehler und Nennen (Si5351: 'b' und 'c')

#print ("x = ", x)       # Nachkommastellen von eingegebener PLLNA

print ("\n\nRESULTATE PLL A DIVIDER")
print ("-----------------------")
print ("Nachbildung durch ganzzahligen Bruch a + b/c \ngemaess 'SKYWORKS Application Note AN619':")
print("a = ", a)        # Variable a gem. Si5351-Handbuch
print ("b = ", b)       # Variable b gem. Si5351-Handbuch
print ("c = ", c)       # Variable c gem. Si5351-Handbuch

MSNA_P1 = 128*a+math.floor(128*b/c)-512 #Variable MSNA_P1 gem. Formel Si5351-Handbuch
MSNA_P2 = 128*b-c*math.floor(128*b/c)   #Variable MSNA_P2 gem. Formel Si5351-Handbuch
MSNA_P3 = c                             #Variable MSNA_P3 gem. Formel Si5351-Handbuch

print ("MSNA_P1 = ", MSNA_P1)
print ("MSNA_P2 = ", MSNA_P2)
print ("MSNA_P3 = ", MSNA_P3)

MSNA_P1_regBytes = MSNA_P1.to_bytes(4, 'little') #MSNA_P1 aufgeteilt in 4 Bytes
MSNA_P2_regBytes = MSNA_P2.to_bytes(4, 'little') #MSNA_P2 aufgeteilt in 4 Bytes
MSNA_P3_regBytes = MSNA_P3.to_bytes(4, 'little') #MSNA_P3 aufgeteilt in 4 Bytes



Si5351_Register_30 = MSNA_P1_regBytes[0] 
Si5351_Register_29 = MSNA_P1_regBytes[1] 
Si5351_Register_28 = MSNA_P1_regBytes[2]

Si5351_Register_33 = MSNA_P2_regBytes[0] 
Si5351_Register_32 = MSNA_P2_regBytes[1] 

Si5351_Register_27 = MSNA_P3_regBytes[0] 
Si5351_Register_26 = MSNA_P3_regBytes[1] 

# Si5351-Register 31
# ist Kombination aus MSNA_P2_regByte(2] und MSNA_P3_regByte(2]  -->
Si5351_Register_31 = 0
Si5351_Register_31 |= MSNA_P2_regBytes[2] & 0x0F         #MSNA_P2-regByte Bit 16-19 in das niederwertige Halbbyte von Register 31
Si5351_Register_31 |= (MSNA_P3_regBytes[2] <<4)&0xF0  #MSNA_P3-regByte Bit 16-19 in das hoeherwertige Halbbyte von Register 31


print ("\nRegister-Inhalt:")
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(26, Si5351_Register_26))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(27, Si5351_Register_27))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(28, Si5351_Register_28))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(29, Si5351_Register_29))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(30, Si5351_Register_30))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(31, Si5351_Register_31))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(32, Si5351_Register_32))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(33, Si5351_Register_33))



"""
************************************************
M U L T I S Y N T H  O U T P U T  D I V I D E R 
Berechnung
a, b, c  von MS0
MS0_P1, MS0-P2, MS0_P3
Werte fuer MS0-Register Si5351-Register 42 - 49
************************************************
"""


a = math.floor(MS_DIV)

x = MS_DIV - a          #x = Nachkommastellen

(b, c) =  farey (x)     #Funktionsaufruf zur Berechnung von MSNA - Bruch, Zaehler und Nenner (Si5351-Application Note AN619: 'b' und 'c')


print ("\n\n\nRESULTATE MULTISYNTH OUTPUT DIVIDER")
print ("-----------------------------------")
print ("Nachbildung durch ganzzahligen Bruch a + b/c \ngemaess 'SKYWORKS Application Note AN619':")
print("a = ", a)        # MS-DIV Variable a gem. Si5351-Handbuch
print ("b = ", b)       # MS-DIV Variable b gem. Si5351-Handbuch
print ("c = ", c)       # MS-DIV Variable c gem. Si5351-Handbuch

MS0_P1 = 128*a+math.floor(128*b/c)-512 #Variable MS0_P1 gem. Formel Si5351-Handbuch
MS0_P2 = 128*b-c*math.floor(128*b/c)   #Variable MS0_P2 gem. Formel Si5351-Handbuch
MS0_P3 = c 

print ("MSx_P1 = ", MS0_P1)
print ("MSx_P2 = ", MS0_P2)
print ("MSx_P3 = ", MS0_P3)

MS0_P1_regBytes = MS0_P1.to_bytes(4, 'little') #MS0_P1 aufgeteilt in 4 Bytes
MS0_P2_regBytes = MS0_P2.to_bytes(4, 'little') #MS0_P2 aufgeteilt in 4 Bytes
MS0_P3_regBytes = MS0_P3.to_bytes(4, 'little') #MS0_P3 aufgeteilt in 4 Bytes

Si5351_Register_46 = MS0_P1_regBytes[0] 
Si5351_Register_45 = MS0_P1_regBytes[1] 
Si5351_Register_28 = MS0_P1_regBytes[2]

Si5351_Register_49 = MS0_P2_regBytes[0] 
Si5351_Register_48 = MS0_P2_regBytes[1] 

Si5351_Register_43 = MS0_P3_regBytes[0] 
Si5351_Register_42 = MS0_P3_regBytes[1]


# Si5351-Register 47
# ist Kombination aus MS0_P2_regByte(2] und MS0_P3_regByte(2]  -->
Si5351_Register_47 = 0
Si5351_Register_47 |= MS0_P2_regBytes[2] & 0x0F      #MS0_P2-regByte Bit 16-19 in das niederwertige Halbbyte von Register 47
Si5351_Register_47 |= (MS0_P3_regBytes[2] <<4) & 0xF0  #MS0_P3-regByte Bit 16-19 in das hoeherwertige Halbbyte von Register 47

# Si5351-Register 44 ist Kombination aus MS0_P1_regByte(2] und 
# Wert vom R0-(Output Stage)Divider  -->

Si5351_Register_44 = 0
Si5351_Register_44 |= MS0_P1_regBytes[2]    # MS0_P1-regBytes Bit 16-17 ins Register 44 [1:0]
Si5351_Register_44 |= R_DIV_Bits            # Wert R-DIV = entspechend dem Wert aus Kommandozeile (Skript-Aufruf) [Bits 6:4]

 
print ("\nRegister-Inhalt:")
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+42, Si5351_Register_42))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+43, Si5351_Register_43))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+44, Si5351_Register_44))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+45, Si5351_Register_45))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+46, Si5351_Register_46))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+47, Si5351_Register_47))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+48, Si5351_Register_48))
print ("Register {0:d} dez: {1:03d}, hex: {1:02X}".format(CLK*8+49, Si5351_Register_49))

